﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Threading;  // for sleep functions to throttle timing to board

using AIOPC104Net;  // The namespace exposes the AIOPC104 Class interface (AIO167.dll)

namespace Sample0
{
    public class AIO16A
    {   
        // C# Class Library of converted C functions for this board:
        public Int32 CardNum = 0;
        public UInt32 BASE = 0x300;

        public UInt16 jumpers;
        public UInt16 adrange;
        public UInt16 dacBrange;
        public UInt16 dacArange;

        
        //sbitd takes data and bit, then returns 0x81 or 0x01 based on data[bit]
        public Byte sbitd(UInt32 data, UInt16 bit)
        {
            UInt16 retval; 
            retval = (UInt16)((((data) & (1 << bit)) == 0x81) ? (UInt16) 0x81 : 0x01);
            return ((Byte)retval);       
        }

        public double DAC(UInt16 DACnum, double DACV)
        {
            DACnum &= 1;
          
            int count = (int) (DACV/(( ((DACnum != 0)?dacBrange:dacArange))!=0?5.0:10.0) * 0x1000);
            if (count > 0xFFF)
            count = 0xFFF;
            else if (count < 0)
            count = 0;

            AIOPC104.OutPort((uint)(BASE + 0xC + (DACnum*2)), (ushort) count);
            //outport(BASE + 0x08, 0x8000);

            return (double)(count)/4096.0*(( ((DACnum!=0)?dacArange:dacBrange))!=0?5.0:10.0);//actual V
        }
        
        public void EEPROM_Enable()
        {
            AIOPC104.OutPortB(BASE+0x18,0x80);//start process
            AIOPC104.OutPortB(BASE+0x18,0x81);//start bit
            AIOPC104.OutPortB(BASE+0x18,0x01);//Opcode bit 0 (00 = enable/disable)
            AIOPC104.OutPortB(BASE+0x18,0x01);//Opcode bit 1  ""
            AIOPC104.OutPortB(BASE+0x18,0x81);//a5 (must be 1 to enable)
            AIOPC104.OutPortB(BASE+0x18,0x81);//a4 (must be 1 to enable)
            AIOPC104.OutPortB(BASE+0x18,0x01);//a3 (don't care because enable/disable)
            AIOPC104.OutPortB(BASE+0x18,0x01);//a2 (don't care because enable/disable)
            AIOPC104.OutPortB(BASE+0x18,0x01);//a1 (don't care because enable/disable)
            AIOPC104.OutPortB(BASE+0x18,0x01);//a0 (don't care because enable/disable)
            AIOPC104.OutPortB(BASE+0x18,0x00);//end trans
        }

        public void EEPROM_Disable()
        {
            AIOPC104.OutPortB(BASE+0x18,0x80);//start process
            AIOPC104.OutPortB(BASE+0x18,0x81);//start bit
            AIOPC104.OutPortB(BASE+0x18,0x01);//Opcode bit 0 (00 = enable/disable)
            AIOPC104.OutPortB(BASE+0x18,0x01);//Opcode bit 1  ""
            AIOPC104.OutPortB(BASE+0x18,0x01);//a5 (must be 0 to disable)
            AIOPC104.OutPortB(BASE+0x18,0x01);//a4 (must be 0 to disable)
            AIOPC104.OutPortB(BASE+0x18,0x01);//a3 (don't care because enable/disable)
            AIOPC104.OutPortB(BASE+0x18,0x01);//a2 (don't care because enable/disable)
            AIOPC104.OutPortB(BASE+0x18,0x01);//a1 (don't care because enable/disable)
            AIOPC104.OutPortB(BASE+0x18,0x01);//a0 (don't care because enable/disable)
            AIOPC104.OutPortB(BASE+0x18,0x00);//end trans
        }

        public void EEPROM_Write(uint addr,uint data)
        {
            AIOPC104.OutPortB(BASE+0x18,0x80);//start process
            AIOPC104.OutPortB(BASE+0x18,0x81);//Start Bit
            AIOPC104.OutPortB(BASE+0x18,0x01);//Opcode Bit 0 (01=write)
            AIOPC104.OutPortB(BASE+0x18,0x81);//Opcode Bit 1  ""
          
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,5));//a5 (address to write to)
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,4));//a4
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,3));//a3
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,2));//a2
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,1));//a1
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,0));//a0
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,15));//d15
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,14));//d14
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,13));//d13
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,12));//d12
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,11));//d11
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,10));//d10
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,9));//d9
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,8));//d8
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,7));//d7
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,6));//d6
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,5));//d5
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,4));//d4
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,3));//d3
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,2));//d2
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,1));//d1
            AIOPC104.OutPortB(BASE+0x18,sbitd(data,0));//d0
            AIOPC104.OutPortB(BASE+0x18,0x00);//end trans

            Thread.Sleep(20); ;
        }

        public uint EEPROM_Read(UInt32 addr)        
        {          
            UInt32 data=0;
          
            AIOPC104.OutPortB(BASE+0x18,0x80);//start process
            AIOPC104.OutPortB(BASE+0x18,0x81);//Start Bit
            AIOPC104.OutPortB(BASE+0x18,0x81);//Opcode Bit 0 (10=Read)
            AIOPC104.OutPortB(BASE+0x18,0x01);//Opcode Bit 1  ""
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,5));//a5 (address to write to)
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,4));//a4
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,3));//a3
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,2));//a2
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,1));//a1
            AIOPC104.OutPortB(BASE+0x18,sbitd(addr,0));//a0        

            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d15
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d14
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d13
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d12
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d11
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d10
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d9
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d8
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d7
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d6
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d5
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d4
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d3
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d2
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d1
            data = (data << 1) | (UInt16)((((AIOPC104.InPortB(BASE + 0x18)) & 0x80) == 0x80) ? (UInt16)1 : 0);//d0

            AIOPC104.OutPortB(BASE+0x18,0x00);//end com
          
            return data;
        }

        public void CAL_ADC(Byte M, Byte B)
        { 
            //write cal to A/D, offset first, then slope
            AIOPC104.OutPortB(BASE+0x19,0x80);//start process
            AIOPC104.OutPortB(BASE+0x19,0x01);//a1 pot 00 is offset
            AIOPC104.OutPortB(BASE+0x19,0x01);//a0 ""
            AIOPC104.OutPortB(BASE+0x19,sbitd(B,7));//data bit 7 of offset
            AIOPC104.OutPortB(BASE+0x19,sbitd(B,6));//data bit 6 of offset
            AIOPC104.OutPortB(BASE+0x19,sbitd(B,5));//data bit 5 of offset
            AIOPC104.OutPortB(BASE+0x19,sbitd(B,4));//data bit 4 of offset
            AIOPC104.OutPortB(BASE+0x19,sbitd(B,3));//data bit 3 of offset
            AIOPC104.OutPortB(BASE+0x19,sbitd(B,2));//data bit 2 of offset
            AIOPC104.OutPortB(BASE+0x19,sbitd(B,1));//data bit 1 of offset
            AIOPC104.OutPortB(BASE+0x19,sbitd(B,0));//data bit 0 of offset
            AIOPC104.OutPortB(BASE+0x19,0x00);//end trans

            Thread.Sleep(1); 

            //write slope now
            AIOPC104.OutPortB(BASE+0x19,0x80);//start process
            AIOPC104.OutPortB(BASE+0x19,0x01);//a1 pot 01 is slope of A/D
            AIOPC104.OutPortB(BASE+0x19,0x81);//a0 pot 01 is slope of A/D
            AIOPC104.OutPortB(BASE+0x19,sbitd(M,7));//data bit 7 of slope
            AIOPC104.OutPortB(BASE+0x19,sbitd(M,6));//data bit 6 of slope
            AIOPC104.OutPortB(BASE+0x19,sbitd(M,5));//data bit 5 of slope
            AIOPC104.OutPortB(BASE+0x19,sbitd(M,4));//data bit 4 of slope
            AIOPC104.OutPortB(BASE+0x19,sbitd(M,3));//data bit 3 of slope
            AIOPC104.OutPortB(BASE+0x19,sbitd(M,2));//data bit 2 of slope
            AIOPC104.OutPortB(BASE+0x19,sbitd(M,1));//data bit 1 of slope
            AIOPC104.OutPortB(BASE+0x19,sbitd(M,0));//data bit 0 of slope
            AIOPC104.OutPortB(BASE+0x19,0x00);//end trans
        }

        public void CAL_DAC(Byte DACNum, Byte M)
        {
            //PCICARDs use +1A instead of +19, and the A1 bit needs to be cleared
            //we can read BASE+0x1F to determine if its the PCI card or not.  PCI==0
            //unsigned PCICARD=(InPortB(BASE+0x1F)==0x00) ? 1 : 0;
            UInt16 PCICARD = (ushort)((AIOPC104.InPortB(BASE + 0x1F) == 0x00) ? 1 : 0);

            AIOPC104.OutPortB(BASE+0x19+PCICARD,0x80);//start process
            AIOPC104.OutPortB((uint)(BASE+0x19+PCICARD),(byte)( (PCICARD!=0)?0x01:0x81) );
            AIOPC104.OutPortB(BASE+0x19+PCICARD,sbitd(DACNum,0));//a0 pot 11 is slope of DAC1
            AIOPC104.OutPortB(BASE+0x19+PCICARD,sbitd(M,7));//data bit 7 of slope
            AIOPC104.OutPortB(BASE+0x19+PCICARD,sbitd(M,6));//data bit 6 of slope
            AIOPC104.OutPortB(BASE+0x19+PCICARD,sbitd(M,5));//data bit 5 of slope
            AIOPC104.OutPortB(BASE+0x19+PCICARD,sbitd(M,4));//data bit 4 of slope
            AIOPC104.OutPortB(BASE+0x19+PCICARD,sbitd(M,3));//data bit 3 of slope
            AIOPC104.OutPortB(BASE+0x19+PCICARD,sbitd(M,2));//data bit 2 of slope
            AIOPC104.OutPortB(BASE+0x19+PCICARD,sbitd(M,1));//data bit 1 of slope
            AIOPC104.OutPortB(BASE+0x19+PCICARD,sbitd(M,0));//data bit 0 of slope
            AIOPC104.OutPortB(BASE+0x19+PCICARD,0x00);//end trans
        }

        public void CALWrite(UInt32 zAM, UInt32 zAB, UInt32 zDAM, UInt32 zDBM)
        {
            //EEPROM_Enable();
            EEPROM_Write((uint)(adrange+0),zAB);
            EEPROM_Write((uint)(adrange+8),zAM);
            EEPROM_Write((uint)(dacArange+0x10),zDAM);
            EEPROM_Write((uint)(dacBrange+0x12),zDBM);
            //EEPROM_Disable();
        }

        public void doCAL()
        {
            UInt16 zDAM, zDBM, zAM, zAB;

            zAB = (UInt16)(EEPROM_Read((uint)(adrange+0))&0xff);  //byte 0-7 in EEPROM holds "B" for ADC
            zAM = (UInt16)(EEPROM_Read((uint)(adrange+8))&0xff);  //byte 8-f in EEPROM holds "M" for ADC
            zDAM = (UInt16)(EEPROM_Read((uint)(dacArange+0x10))&0xff); //byte x10,x11 in EEPROM holds "M" for DAC0
            zDBM = (UInt16)(EEPROM_Read((uint)(dacBrange+0x12))&0xff); //byte x12,x13 in EEPROM holds "M" for DAC1
            CAL_DAC(0,(byte)zDAM);//write the calibration value from EEPROM to CAL Pots
            CAL_DAC(1,(byte)zDBM);// ""
            CAL_ADC((byte)zAM,(byte)zAB);// ""
        }

        public void init()
        {
            jumpers=AIOPC104.InPortB(BASE+0x12);//jumper configuration readback
            adrange= (UInt16)(jumpers & 0x07);//d2,d1,d0 are 5/10, uni/bip, 16/8
            dacBrange= (UInt16)((jumpers>>3) & 0x01);//d4,d3 are 5/10 for daca,b
            dacArange= (UInt16)((jumpers>>4) & 0x01);
            //CAL();
        }

    }
}
